@@ -170,12 +170,6 @@ static void               launcher_plugin_item_exec                     (GarconM
 static void               launcher_plugin_item_exec_from_clipboard      (GarconMenuItem       *item,
                                                                          guint32               event_time,
                                                                          GdkScreen            *screen);
-static void               launcher_plugin_exec_append_quoted            (GString              *string,
-                                                                         const gchar          *unquoted);
-static gboolean           launcher_plugin_exec_parse                    (GarconMenuItem       *item,
-                                                                         GSList               *uri_list,
-                                                                         gchar              ***argv,
-                                                                         GError              **error);
 static GSList            *launcher_plugin_uri_list_extract              (GtkSelectionData     *data);
 static void               launcher_plugin_uri_list_free                 (GSList               *uri_list);
 
@@ -1503,8 +1497,15 @@ launcher_plugin_menu_deactivate (GtkWidget      *menu,
   panel_return_if_fail (plugin->menu == menu);
 
   /* deactivate the arrow button */
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->arrow), FALSE);
-  gtk_widget_unset_state_flags (GTK_WIDGET (plugin->arrow), GTK_STATE_FLAG_PRELIGHT);
+  if (plugin->arrow_position != LAUNCHER_ARROW_INTERNAL)
+    {
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->arrow), FALSE);
+      gtk_widget_unset_state_flags (GTK_WIDGET (plugin->arrow), GTK_STATE_FLAG_PRELIGHT);
+    }
+  else
+    {
+      gtk_widget_set_state_flags (GTK_WIDGET (plugin->button), GTK_STATE_FLAG_NORMAL, TRUE);
+    }
 }
 
 
@@ -1594,9 +1595,16 @@ launcher_plugin_menu_item_drag_data_received (GtkWidget          *widget,
   gtk_widget_hide (gtk_widget_get_toplevel (plugin->menu));
   gtk_widget_hide (plugin->menu);
 
-  /* inactivate the toggle button */
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->arrow), FALSE);
-  gtk_widget_unset_state_flags (GTK_WIDGET (plugin->arrow), GTK_STATE_FLAG_PRELIGHT);
+  /* deactivate the toggle button */
+  if (plugin->arrow_position != LAUNCHER_ARROW_INTERNAL)
+    {
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->arrow), FALSE);
+      gtk_widget_unset_state_flags (GTK_WIDGET (plugin->arrow), GTK_STATE_FLAG_PRELIGHT);
+    }
+  else
+    {
+      gtk_widget_set_state_flags (GTK_WIDGET (plugin->button), GTK_STATE_FLAG_NORMAL, TRUE);
+    }
 
   /* finish the drag */
   gtk_drag_finish (context, TRUE, FALSE, drag_time);
@@ -1721,7 +1729,10 @@ launcher_plugin_menu_popup (gpointer user_data)
     launcher_plugin_menu_construct (plugin);
 
   /* toggle the arrow button */
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->arrow), TRUE);
+  if (plugin->arrow_position != LAUNCHER_ARROW_INTERNAL)
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->arrow), TRUE);
+  else
+    gtk_widget_set_state_flags (GTK_WIDGET (plugin->button), GTK_STATE_FLAG_CHECKED, TRUE);
 
   /* popup the menu */
   gtk_menu_popup_at_widget (GTK_MENU (plugin->menu),
@@ -1770,8 +1781,15 @@ launcher_plugin_menu_destroy (LauncherPlugin *plugin)
       plugin->menu = NULL;
 
       /* deactivate the toggle button */
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->arrow), FALSE);
-      gtk_widget_unset_state_flags (GTK_WIDGET (plugin->arrow), GTK_STATE_FLAG_PRELIGHT);
+      if (plugin->arrow_position != LAUNCHER_ARROW_INTERNAL)
+        {
+          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->arrow), FALSE);
+          gtk_widget_unset_state_flags (GTK_WIDGET (plugin->arrow), GTK_STATE_FLAG_PRELIGHT);
+        }
+      else
+        {
+          gtk_widget_set_state_flags (GTK_WIDGET (plugin->button), GTK_STATE_FLAG_NORMAL, TRUE);
+        }
     }
 }
 
@@ -1902,9 +1920,9 @@ launcher_plugin_button_update_action_menu (LauncherPlugin *plugin)
   else
     {
       plugin->action_menu = GTK_WIDGET (garcon_gtk_menu_get_desktop_actions_menu (item));
-      gtk_menu_set_reserve_toggle_size (GTK_MENU (plugin->action_menu), FALSE);
       if (plugin->action_menu)
         {
+          gtk_menu_set_reserve_toggle_size (GTK_MENU (plugin->action_menu), FALSE);
           gtk_container_foreach (GTK_CONTAINER (plugin->action_menu),
                                  launcher_plugin_add_desktop_actions,
                                  plugin);
@@ -2454,24 +2472,37 @@ launcher_plugin_item_exec_on_screen (GarconMenuItem *item,
                                      GdkScreen      *screen,
                                      GSList         *uri_list)
 {
-  GError    *error = NULL;
-  gchar    **argv;
-  gboolean   succeed = FALSE;
+  GError      *error = NULL;
+  gchar      **argv;
+  gboolean     succeed = FALSE;
+  gchar       *command, *uri;
+  const gchar *icon;
 
   panel_return_val_if_fail (GARCON_IS_MENU_ITEM (item), FALSE);
   panel_return_val_if_fail (GDK_IS_SCREEN (screen), FALSE);
 
+  /* get the command */
+  command = (gchar*) garcon_menu_item_get_command (item);
+  panel_return_val_if_fail (!panel_str_is_empty (command), FALSE);
+
+  /* expand the field codes */
+  icon = garcon_menu_item_get_icon_name (item);
+  uri = garcon_menu_item_get_uri (item);
+  command = xfce_expand_desktop_entry_field_codes (command, uri_list, icon,
+                                                   garcon_menu_item_get_name (item),
+                                                   uri,
+                                                   garcon_menu_item_requires_terminal (item));
+  g_free (uri);
+
   /* parse the execute command */
-  if (launcher_plugin_exec_parse (item, uri_list, &argv, &error))
+  if (g_shell_parse_argv (command, NULL, &argv, &error))
     {
       /* launch the command on the screen */
       succeed = xfce_spawn_on_screen (screen,
                                       garcon_menu_item_get_path (item),
                                       argv, NULL, G_SPAWN_SEARCH_PATH,
                                       garcon_menu_item_supports_startup_notification (item),
-                                      event_time,
-                                      garcon_menu_item_get_icon_name (item),
-                                      &error);
+                                      event_time, icon, &error);
 
       g_strfreev (argv);
     }
@@ -2479,12 +2510,12 @@ launcher_plugin_item_exec_on_screen (GarconMenuItem *item,
   if (G_UNLIKELY (!succeed))
     {
       /* show an error dialog */
-      xfce_dialog_show_error (NULL, error,
-                              _("Failed to execute command \"%s\"."),
-                              garcon_menu_item_get_command (item));
+      xfce_dialog_show_error (NULL, error, _("Failed to execute command \"%s\"."), command);
       g_error_free (error);
     }
 
+  g_free (command);
+
   return succeed;
 }
 
@@ -2578,123 +2609,6 @@ launcher_plugin_item_exec_from_clipboard (GarconMenuItem *item,
 
 
 
-static void
-launcher_plugin_exec_append_quoted (GString     *string,
-                                    const gchar *unquoted)
-{
-  gchar *quoted;
-
-  quoted = g_shell_quote (unquoted);
-  g_string_append (string, quoted);
-  g_free (quoted);
-}
-
-
-
-static gboolean
-launcher_plugin_exec_parse (GarconMenuItem   *item,
-                            GSList           *uri_list,
-                            gchar          ***argv,
-                            GError          **error)
-{
-  GString     *string;
-  const gchar *p;
-  gboolean     result;
-  GSList      *li;
-  gchar       *filename, *uri;
-  const gchar *command, *tmp;
-
-  panel_return_val_if_fail (GARCON_IS_MENU_ITEM (item), FALSE);
-
-  /* get the command */
-  command = garcon_menu_item_get_command (item);
-  panel_return_val_if_fail (!panel_str_is_empty (command), FALSE);
-
-  /* allocate an empty string */
-  string = g_string_sized_new (100);
-
-  /* prepend terminal command if required */
-  if (garcon_menu_item_requires_terminal (item))
-    g_string_append (string, "exo-open --launch TerminalEmulator ");
-
-  for (p = command; *p != '\0'; ++p)
-    {
-      if (G_UNLIKELY (p[0] == '%' && p[1] != '\0'))
-        {
-          switch (*++p)
-            {
-            case 'f':
-            case 'F':
-              for (li = uri_list; li != NULL; li = li->next)
-                {
-                  filename = g_filename_from_uri ((const gchar *) li->data,
-                                                  NULL, NULL);
-                  if (G_LIKELY (filename != NULL))
-                    launcher_plugin_exec_append_quoted (string, filename);
-                  g_free (filename);
-
-                  if (*p == 'f')
-                    break;
-                  if (li->next != NULL)
-                    g_string_append_c (string, ' ');
-                }
-              break;
-
-            case 'u':
-            case 'U':
-              for (li = uri_list; li != NULL; li = li->next)
-                {
-                  launcher_plugin_exec_append_quoted (string, (const gchar *)
-                                                      li->data);
-
-                  if (*p == 'u')
-                    break;
-                  if (li->next != NULL)
-                    g_string_append_c (string, ' ');
-                }
-              break;
-
-            case 'i':
-              tmp = garcon_menu_item_get_icon_name (item);
-              if (!panel_str_is_empty (tmp))
-                {
-                  g_string_append (string, "--icon ");
-                  launcher_plugin_exec_append_quoted (string, tmp);
-                }
-              break;
-
-            case 'c':
-              tmp = garcon_menu_item_get_name (item);
-              if (!panel_str_is_empty (tmp))
-                launcher_plugin_exec_append_quoted (string, tmp);
-              break;
-
-            case 'k':
-              uri = garcon_menu_item_get_uri (item);
-              if (!panel_str_is_empty (uri))
-                launcher_plugin_exec_append_quoted (string, uri);
-              g_free (uri);
-              break;
-
-            case '%':
-              g_string_append_c (string, '%');
-              break;
-            }
-        }
-      else
-        {
-          g_string_append_c (string, *p);
-        }
-    }
-
-  result = g_shell_parse_argv (string->str, NULL, argv, error);
-  g_string_free (string, TRUE);
-
-  return result;
-}
-
-
-
 static GSList *
 launcher_plugin_uri_list_extract (GtkSelectionData *data)
 {
